/*
** queue.c - FIFO queue management routines.
**
** Copyright (c) 1997-1999 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "config.h"

#include <stdlib.h>

#include "plib/safestr.h"
#include "plib/queue.h"



int
pqueue_init(PQUEUE *qp,
	   int qsize)
{
    qp->buf = s_malloc(sizeof(void *) * qsize);

    qp->qsize = qsize;
    qp->occupied = 0;
    qp->nextin = 0;
    qp->nextout = 0;

    pthread_mutex_init(&qp->mtx, NULL);
    pthread_cond_init(&qp->more, NULL);
    pthread_cond_init(&qp->less, NULL);

    return 0;
}



void
pqueue_put(PQUEUE *qp,
	  void *item)
{
    pthread_mutex_lock(&qp->mtx);

    while (qp->occupied >= qp->qsize)
	pthread_cond_wait(&qp->less, &qp->mtx);

    qp->buf[qp->nextin++] = item;

    qp->nextin %= qp->qsize;
    qp->occupied++;

    pthread_mutex_unlock(&qp->mtx);
    pthread_cond_signal(&qp->more);
}



void *
pqueue_get(PQUEUE *qp)
{
    void *item;

    
    pthread_mutex_lock(&qp->mtx);
    
    while (qp->occupied <= 0)
	pthread_cond_wait(&qp->more, &qp->mtx);

    item = qp->buf[qp->nextout++];
    qp->nextout %= qp->qsize;
    qp->occupied--;

    pthread_mutex_unlock(&qp->mtx);
    pthread_cond_signal(&qp->less);

    return item;
}



void
pqueue_destroy(PQUEUE *qp)
{
    pthread_mutex_destroy(&qp->mtx);
    pthread_cond_destroy(&qp->more);
    pthread_cond_destroy(&qp->less);
    s_free(qp->buf);
}
