/*
** readdir.c - Raw directory read functions
**
** Copyright (c) 1995-1999 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "config.h"

#include <stdio.h>
#include <errno.h>

#include <fcntl.h>
#include <limits.h>
#include <stdlib.h>

#include "plib/threads.h"
#include "plib/safeio.h"
#include "plib/safestr.h"
#include "plib/readdir.h"


#ifndef HAVE_GETDENTS
static pthread_mutex_t readdir_lock;
static pthread_once_t readdir_once = PTHREAD_ONCE_INIT;

static void
readdir_lock_init(void)
{
    pthread_mutex_init(&readdir_lock, NULL);
}

struct dirent *
s_readdir(S_DIR *dp)
{
    int rval;
    struct dirent *dep, *rdep;


    pthread_once(&readdir_once, readdir_lock_init);

    pthread_mutex_lock(&readdir_lock);
    dep = readdir(dp);
    if (dep != NULL)
    {
        rdep = s_malloc(sizeof(*dep)+strlen(dep->d_name));
        memcpy(rdep, dep, sizeof(*dep)+strlen(dep->d_name));
	dep = rdep;
    }
    pthread_mutex_unlock(&readdir_lock);
    return dep;
}


#else

S_DIR *
s_opendir(const char *path)
{
    int fd;
    S_DIR *cb;

    
    fd = s_open(path, O_RDONLY);
    if (fd < 0)
	return NULL;
    
    cb = s_malloc(sizeof(S_DIR));
    cb->fd = fd;

    return cb;
}


struct dirent *
s_readdir(S_DIR *dp)
{
    struct dirent *rdep;
    int len;
    

    if (dp->len == 0)
    {
	while ((dp->len = getdents(dp->fd,
				  (struct dirent *) dp->buf,
				  sizeof(dp->buf))) < 0 && errno == EINTR)
	    ;
	dp->dep = (struct dirent *) dp->buf;
    }

    if (dp->len <= 0)
	return NULL;

    len = strlen(dp->dep->d_name);
    rdep = s_malloc(sizeof(struct dirent) + len);
    *rdep = *(dp->dep);
    memcpy(rdep->d_name, dp->dep->d_name, len+1);
    rdep->d_reclen = sizeof(struct dirent) + len;
    
    dp->len -= dp->dep->d_reclen;

    dp->dep = (struct dirent *) (((char *) dp->dep) + dp->dep->d_reclen);

    return rdep;
}

void
s_closedir(S_DIR *dp)
{
    s_close(dp->fd);
    s_free(dp);
}

#endif
