/*
** safestr.h - Buffer overrun safe string functions
**
** Copyright (c) 1997-1999 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef PLIB_SAFESTR_H
#define PLIB_SAFESTR_H

#include <stdarg.h>
#include <sys/types.h>

extern void *
s_malloc(size_t size);

extern void *
s_realloc(void *p,
	  size_t size);

extern void
s_free(void *p);

extern char *
s_strdup(const char *s);

extern char *
s_strcpy(char *to,
	 size_t tolen,
	 const char *from);

extern char *
s_strcat(char *to,
	 size_t tolen,
	 const char *from);

extern char *
s_strtok_r(char *b,
	   const char *s,
	   char **bp);

extern int
s_strcasecmp(const char *s1,
	     const char *s2);

extern int
s_vsnprintf(char *buf,
	    size_t bufsize,
	    const char *format,
	    va_list ap);


extern int
s_snprintf(char *buf,
	   size_t bufsize,
	   const char *format, ...);

#ifndef IN_SAFESTR_C
#define malloc #error
#define realloc #error
#define free #error
#define strdup #error
#define strcpy #error
#define strcat #error
#define strtok #error
#define strtok_r #error
#define strcasecmp #error
#define vsprintf #error
#define vsnprintf #error
#define sprintf #error
#define snprintf #error
#endif

#endif
