/*
** system.h - Compatibility definitions for various systems.
**
** Copyright (c) 1998 Rainer Orth <ro@TechFak.Uni-Bielefeld.DE>
**		 1997 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef PLIB_SYSTEM_H
#define PLIB_SYSTEM_H

#ifdef INET6
# include <sys/types.h>
# include <netinet/in.h>
# ifdef SIN6_LEN
#  define SA_LEN(a)	(a)->su_si.si_len
# else
#  ifdef linux
#   include <support.h>
#   ifndef SA_LEN
#    define SA_LEN(a)	NRL_SA_LEN(a)
#   endif
# endif
#endif

union sockunion {
    struct sockinet {
#ifdef SIN6_LEN
	u_char si_len;
	u_char si_family;
#else
	sa_family_t si_family;
#endif
	u_short si_port;
    } su_si;
    struct sockaddr_in  su_sin;
    struct sockaddr_in6 su_sin6;
};
#ifdef SIN6_LEN
#define su_len		su_si.si_len
#endif
#define su_family	su_si.si_family
#define su_port		su_si.si_port
#define su_sin_addr	su_sin.sin_addr
#define su_sin6_addr	su_sin6.sin6_addr
#endif

#ifndef F_OK
#define F_OK 0
#endif

#ifndef INADDR_LOOPBACK
#define INADDR_LOOPBACK 0x7F000001U
#endif

#ifndef LOG_ODELAY
#define LOG_ODELAY 0
#endif

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#endif
